#include "FAtoLRGConverter.h"

using namespace std;
using namespace automaton;
using namespace grammar;
using namespace alib;

namespace conversions
{

FAtoLRGConverter::FAtoLRGConverter( const FSM & automaton ) : m_automaton( automaton )
{
    if( ! automaton.isEpsilonFree( ) )
        throw AlibException( "FSM has epsilon transitions." );
}

FAtoLRGConverter::~FAtoLRGConverter( void )
{

}

LeftRegularGrammar FAtoLRGConverter::convert( void )
{
    LeftRegularGrammar grammar;
    map<State, Symbol> nonterminalMap;

    for( const auto & symbol : m_automaton.getInputAlphabet( ) )
        grammar.addTerminalSymbol( symbol );

    for( const auto & state : m_automaton.getStates( ) )
    {
        Symbol nt = grammar.createUniqueNonTerminalSymbol( state.getName( ) );
        nonterminalMap.insert( pair<State,Symbol>( state, nt ) );
    }

    // step 2
    Symbol newStart = grammar.createUniqueNonTerminalSymbol( "S", false );
    grammar.setStartSymbol( newStart );

    // step 3 - create set of P in G
    for( const auto & transition : m_automaton.getTransitions( ) )
    {
        // 3a
        list<Symbol> leftSide, rightSide;
        leftSide.push_back( nonterminalMap.find( transition.getTo( ) )->second );
        rightSide.push_back( nonterminalMap.find( transition.getFrom( ) )->second );
        rightSide.push_back( transition.getInput( ) );
        grammar.addRule( Rule( leftSide, rightSide ) );

        if( isInSet( transition.getTo( ), m_automaton.getFinalStates( ) ) )
        {
            list<Symbol> leftSide, rightSide;
            leftSide.push_back( grammar.getStartSymbol( ) );
            rightSide.push_back( nonterminalMap.find( transition.getFrom( ) )->second );
            rightSide.push_back( transition.getInput( ) );

            Rule r( leftSide, rightSide );
            if( ! isInSet( r, grammar.getRules( ) ) )
                grammar.addRule( Rule( leftSide, rightSide ) );
        }


        if( isInSet( transition.getFrom( ), m_automaton.getInitialStates( ) ) )
        {
            list<Symbol> leftSide, rightSide;
            leftSide.push_back( nonterminalMap.find( transition.getTo( ) )->second );
            rightSide.push_back( transition.getInput( ) );
            grammar.addRule( Rule( leftSide, rightSide ) );

            if( isInSet( transition.getTo( ), m_automaton.getFinalStates( ) ) )
            {
                list<Symbol> leftSide, rightSide;
                leftSide.push_back( grammar.getStartSymbol( ) );
                rightSide.push_back( transition.getInput( ) );

                Rule r( leftSide, rightSide );
                if( ! isInSet( r, grammar.getRules( ) ) )
                    grammar.addRule( Rule( leftSide, rightSide ) );
            }
        }
    }

    if( isInSet( * m_automaton.getInitialStates( ).begin( ), m_automaton.getFinalStates( ) ) )
    {
        list<Symbol> leftSide = { newStart }, rightSide;
        grammar.addRule( Rule( leftSide, rightSide ) );
    }

    return grammar;
}

} /* namespace conversions */
