#include "RRGtoFAConverter.h"

using namespace alphabet;
using namespace automaton;
using namespace grammar;

namespace conversions
{

RRGtoFAConverter::RRGtoFAConverter( const RightRegularGrammar & grammar ) : m_grammar( grammar )
{

}

RRGtoFAConverter::~RRGtoFAConverter( void )
{

}


FSM RRGtoFAConverter::convert( void )
{
    FSM automaton;

    for( const auto & symbol : m_grammar.getTerminalSymbols( ) )
        automaton.addInputSymbol( symbol );

    for( const auto & symbol : m_grammar.getNonTerminalSymbols( ) )
        automaton.addState( State( symbol.getSymbol( ) ) );

    const State & AState = automaton.createUniqueState( "A", false );

    for( const auto & rule : m_grammar.getRules( ) )
    {
        if( m_grammar.isEpsilonRule( rule ) )
            continue;

        State current( rule.getLeftSide( ).front( ).getSymbol( ) );

        if( rule.getRightSide( ).size( ) == 1 ) // if B->a => \delta(B,a)=AState
        {
            const Symbol & input( rule.getRightSide( ).front( ).getSymbol( ) );
            automaton.addTransition( current, input, AState );
        }
        else if( rule.getRightSide( ).size( ) == 2 ) // if B->aC => \delta(B,a)=C
        {
            const Symbol & input = rule.getRightSide( ).front( );
            State next( rule.getRightSide( ).back( ).getSymbol( ) );

            automaton.addTransition( current, input, next );
        }
    }

    // step 4
    automaton.addInitialState( m_grammar.getStartSymbol().getSymbol() );

    // step 5
    automaton.addFinalState( AState );
    for( const auto & rule : m_grammar.getRules( ) )
        if( m_grammar.isEpsilonRule( rule ) )
            automaton.addFinalState( State( m_grammar.getStartSymbol( ).getSymbol( ) ) );

    return automaton;
}

} /* namespace conversions */
