/*
 * RightRegularEquationSolver.cpp
 *
 *  Created on: 4. 3. 2014
 *      Author: tomas
 */

#include "RightRegularEquationSolver.h"

using namespace alib;
using namespace alphabet;
using namespace regexp;
using namespace std;

namespace conversions
{

RightRegularEquationSolver::RightRegularEquationSolver( void )
{

}

RightRegularEquationSolver::~RightRegularEquationSolver( void )
{
    for( const auto & kv : m_eqFinal )
        delete kv.second;
}

RegExpElement* RightRegularEquationSolver::eliminate( void )
{
    RegExpOptimize opt;

    for( auto itA = m_symbolsByDepth.rbegin( ); itA != m_symbolsByDepth.rend( ); itA ++ )
    {
        const Symbol & a = * itA;

        /*
         * Apply Arden's Lemma
         * A = 0A + 1B + 2C
         * => A = 0*1B + 0*2C
         */
        Iteration* loop = new Iteration( );
        loop->setElement( m_eqTransition[ a ][ a ] );

        // for all transitions from A apply Arden's Lemma
        for( auto itB = std::next( itA ) ; itB != m_symbolsByDepth.rend( ); itB ++ )
        {
            const Symbol & b = * itB;
            Concatenation* concat = new Concatenation( );
            concat->getElements( ).push_back( loop->clone( ) );
            concat->getElements( ).push_back( m_eqTransition[ a ][ b ] );
            m_eqTransition[ a ][ b ] = opt.optimize( concat );
            delete concat;
        }
        Concatenation* concat = new Concatenation( );
        concat->getElements( ).push_back( loop->clone( ) );
        concat->getElements( ).push_back( m_eqFinal[ a ] );
        m_eqFinal[ a ] = opt.optimize( concat );
        delete concat;

        loop->setElement( NULL );
        delete loop;

        /*
         * eliminate A from rest of the equations using this pattern:
         * B->C = B->C + concatenate( B->A, A->C )
         */
        for( auto itB = std::next( itA ); itB != m_symbolsByDepth.rend( ); itB ++ )
        {
            const Symbol & b = * itB;

            for( auto itC = std::next( itA ); itC != m_symbolsByDepth.rend( ); itC ++ )
            {
                const Symbol & c = * itC;

                Alternation* alt = new Alternation( );
                Concatenation* concat = new Concatenation( );
                concat->getElements( ).push_back( m_eqTransition[ b ][ a ]->clone( ) );
                concat->getElements( ).push_back( m_eqTransition[ a ][ c ]->clone( ) );
                alt->getElements( ).push_back( m_eqTransition[ b ][ c ] );
                alt->getElements( ).push_back( concat );
                m_eqTransition[ b ][ c ] = opt.optimize( alt );
                delete alt;
            }

            Alternation* alt = new Alternation( );
            Concatenation* concat = new Concatenation( );
            concat->getElements( ).push_back( m_eqTransition[ b ][ a ]->clone( ) );
            concat->getElements( ).push_back( m_eqFinal[ a ]->clone( ) );
            alt->getElements( ).push_back( m_eqFinal[ b ] );
            alt->getElements( ).push_back( concat );
            m_eqFinal[ b ] = opt.optimize( alt );
            delete alt;
        }

        // no need for this now, release some memory
        for( const auto & kv : m_eqTransition[ a ] )
            delete kv.second;
    }

    RegExpElement* result = m_eqFinal[ * m_symbolsByDepth.begin( ) ];
    m_eqFinal[ * m_symbolsByDepth.begin( ) ] = NULL;

    return result;
}

} /* namespace conversions */
